@@ -1,41 +1,22 @@
 
-#' Hydrate a Project
+#' Copy packages from user libraries to a project library
 #'
-#' Discover the \R packages used within a project, and then install those
-#' packages into the active library. This effectively allows you to fork the
-#' state of your default \R libraries for use within a project library.
+#' @description
+#' `hydrate()` installs missing packages from a user library into the project
+#' library. `hydrate()` is called automatically by [init()], and it is rare
+#' that you should need it otherwise, as it can easily get your project into
+#' an inconsistent state.
 #'
-#' It may occasionally be useful to use `renv::hydrate()` to update the packages
-#' used within a project that has already been initialized. However, be aware
-#' that it's possible that the packages pulled in may not actually be compatible
-#' with the packages already installed in the project library, so you should
-#' exercise caution when doing so.
-#'
-#' @section Sources:
-#'
-#' `hydrate()` attempts to re-use packages already installed on your system,
-#' to avoid unnecessary attempts to download and install packages from remote
-#' sources. When `NULL` (the default), `hydrate()` will attempt to discover \R
-#' packages from the following sources (in order):
-#'
-#' - The user library,
-#' - The site library,
-#' - The system library,
-#' - The `renv` cache.
-#'
-#' If package is discovered in one of these locations, `renv` will attempt to
-#' copy or link that package into the requested library as appropriate.
-#'
-#' @section Missing Packages:
-#'
-#' If `renv` discovers that your project depends on \R packages not currently
-#' installed in your user library, then it will attempt to install those
-#' packages from the active R repositories.
+#' It may very occasionally be useful to call `hydate(update = "all")` if you
+#' want to update project packages to match those installed in your global
+#' library (as opposed to using [update()] which will get the latest versions
+#' from CRAN). In this case, you should verify that your code continues to work,
+#' then call [snapshot()] to record updated package versions in the lockfile.
 #'
 #' @inherit renv-params
 #'
 #' @param packages The set of \R packages to install. When `NULL`, the
-#'   set of packages as reported by [dependencies()] is used.
+#'   packages found by [dependencies()] are used.
 #'
 #' @param library The \R library to be hydrated. When `NULL`, the active
 #'   library as reported by `.libPaths()` is used.
@@ -45,8 +26,13 @@
 #'   library? Set this to `"all"` if you'd like _all_ packages to be refreshed
 #'   from the source library if possible.
 #'
-#' @param sources A set of library paths from which `renv` should attempt to
-#'   draw packages. See **Sources** for more details.
+#' @param sources A vector of library paths where renv should look for packages.
+#'   When `NULL` (the default), `hydrate()` will look in the system libraries
+#'   (the user library, the site library and the default library) then the
+#'   renv cache.
+#'
+#'   If a package is not found in any of these locations, `hydrate()`
+#'   will try to install it from the active R repositories.
 #'
 #' @param prompt Boolean; prompt the user before taking any action? Ignored
 #'   when `report = FALSE`.
@@ -59,6 +45,8 @@
 #'
 #' @export
 #'
+#' @keywords internal
+#'
 #' @examples
 #' \dontrun{
 #'
@@ -94,8 +82,8 @@ hydrate <- function(packages = NULL,
 
   # remove base + missing packages
   base <- renv_packages_base()
-  na <- deps[is.na(deps)]
-  packages <- deps[renv_vector_diff(names(deps), c(names(na), base))]
+  missing <- deps[!nzchar(deps)]
+  packages <- deps[renv_vector_diff(names(deps), c(names(missing), base))]
 
   # figure out if we will copy or link
   linkable <- renv_cache_linkable(project = project, library = library)
@@ -109,19 +97,15 @@ hydrate <- function(packages = NULL,
 
   # inform user about changes
   if (report) {
-    renv_hydrate_report(packages, na, linkable)
-    if (length(packages) || length(na)) {
-      if (prompt && !proceed()) {
-        renv_report_user_cancel()
-        invokeRestart("abort")
-      }
-    }
+    renv_hydrate_report(packages, missing, linkable)
+    if (length(packages) || length(missing))
+      cancel_if(prompt && !proceed())
   }
 
   # check for nothing to be done
-  if (empty(packages) && empty(na)) {
+  if (empty(packages) && empty(missing)) {
     if (report)
-      vwritef("* No new packages were discovered in this project; nothing to do.")
+      writef("- No new packages were discovered in this project; nothing to do.")
     return(invisible(list(packages = list(), missing = list())))
   }
 
@@ -137,12 +121,12 @@ hydrate <- function(packages = NULL,
 
   if (report) {
     time <- difftime(after, before, units = "auto")
-    fmt <- "* Hydrated %s packages in %s."
-    vwritef(fmt, length(packages), renv_difftime_format(time))
+    fmt <- "- Hydrated %s packages in %s."
+    writef(fmt, length(packages), renv_difftime_format(time))
   }
 
   # attempt to install missing packages (if any)
-  missing <- renv_hydrate_resolve_missing(project, library, na)
+  missing <- renv_hydrate_resolve_missing(project, library, missing)
 
   # we're done!
   result <- list(packages = packages, missing = missing)
@@ -202,47 +186,18 @@ renv_hydrate_filter_impl <- function(package, path, library, update) {
 
 }
 
-renv_hydrate_packages_rprofile <- function() {
-
-  enabled <-
-    identical(config$user.profile(), TRUE) &&
-    !renv_tests_running()
-
-  if (!enabled)
-    return()
-
-  rprofile <- Sys.getenv("R_PROFILE_USER", unset = "~/.Rprofile")
-  if (!file.exists(rprofile))
-    return()
-
-  dependencies(rprofile, quiet = TRUE, dev = TRUE)
-
-}
-
 renv_hydrate_packages <- function(project) {
-
-  deps <- dependencies(project, quiet = TRUE, dev = TRUE)
-
-  profdeps <- renv_hydrate_packages_rprofile()
-  if (length(deps) && length(profdeps))
-    deps <- bind(list(deps, profdeps))
-
-  unique(deps$Package)
-
+  renv_snapshot_dependencies(project, dev = TRUE)
 }
 
 renv_hydrate_dependencies <- function(project,
                                       packages = NULL,
                                       libpaths = NULL)
 {
-  vprintf("* Discovering package dependencies ... ")
   ignored <- renv_project_ignored_packages(project = project)
   packages <- renv_vector_diff(packages, ignored)
   libpaths <- libpaths %||% renv_hydrate_libpaths()
-  all <- renv_package_dependencies(packages, project = project, libpaths = libpaths)
-  vwritef("Done!")
-
-  all
+  renv_package_dependencies(packages, libpaths = libpaths, project = project)
 }
 
 # NOTE: we don't want to look in user / site libraries when testing
@@ -255,7 +210,7 @@ renv_hydrate_libpaths <- function() {
     conf <- unlist(strsplit(conf, ":", fixed = TRUE))
 
   libpaths <- case(
-    renv_tests_running() ~ renv_libpaths_all(),
+    renv_tests_running() ~ character(),
     length(conf) ~ conf,
     ~ c(
       renv_libpaths_default(),
@@ -276,7 +231,10 @@ renv_hydrate_libpaths <- function() {
 renv_hydrate_link_package <- function(package, location, library) {
 
   # construct path to cache
-  record <- renv_snapshot_description(location)
+  record <- catch(renv_snapshot_description(location))
+  if (inherits(record, "error"))
+    return(FALSE)
+
   cache <- renv_cache_find(record)
   if (!nzchar(cache))
     return(FALSE)
@@ -296,15 +254,14 @@ renv_hydrate_link_package <- function(package, location, library) {
 
 renv_hydrate_link_packages <- function(packages, library, project) {
 
-  header <- if (renv_path_same(library, renv_paths_library(project = project)))
-    "* Linking packages into the project library ... "
+  if (renv_path_same(library, renv_paths_library(project = project)))
+    printf("- Linking packages into the project library ... ")
   else
-    sprintf("* Linking packages into %s ... ", renv_path_pretty(library))
+    printf("- Linking packages into %s ... ", renv_path_pretty(library))
 
-  vprintf(header)
   callback <- renv_progress_callback(renv_hydrate_link_package, length(packages))
   cached <- enumerate(packages, callback, library = library)
-  vwritef("Done!")
+  writef("Done!")
   cached
 
 }
@@ -318,34 +275,38 @@ renv_hydrate_copy_package <- function(package, location, library) {
 
 renv_hydrate_copy_packages <- function(packages, library, project) {
 
-  header <- if (renv_path_same(library, renv_paths_library(project = project)))
-    "* Copying packages into the project library ... "
+  if (renv_path_same(library, renv_paths_library(project = project)))
+    printf("- Copying packages into the project library ... ")
   else
-    sprintf("* Copying packages into %s ... ", renv_path_pretty(library))
+    printf("- Copying packages into %s ... ", renv_path_pretty(library))
 
-  vprintf(header)
   callback <- renv_progress_callback(renv_hydrate_copy_package, length(packages))
   copied <- enumerate(packages, callback, library = library)
-  vwritef("Done!")
+  writef("Done!")
   copied
 }
 
 renv_hydrate_resolve_missing <- function(project, library, na) {
 
   # make sure requested library is made active
+  #
+  # note that we only want to place the requested library on the library path;
+  # we want to ensure that all required packages are hydrated into the
+  # reqeusted library
+  #
+  # https://github.com/rstudio/renv/issues/1177
   ensure_directory(library)
-  renv_scope_libpaths(unique(c(library, .libPaths())))
+  renv_scope_libpaths(library)
 
   # figure out which packages are missing (if any)
   packages <- names(na)
-  installed <- installed_packages()
+  installed <- installed_packages(lib.loc = library)
   if (all(packages %in% installed$Package))
     return()
 
-  vwritef("* Resolving missing dependencies  ... ")
+  writef("- Resolving missing dependencies ... ")
 
-  # define a custom error handler for packages which
-  # we failed to retrieve
+  # define a custom error handler for packages which we cannot retrieve
   errors <- stack()
   handler <- function(package, action) {
     error <- catch(action)
@@ -379,10 +340,9 @@ renv_hydrate_resolve_missing <- function(project, library, na) {
     })
 
     renv_pretty_print(
-      text,
       "The following package(s) were not installed successfully:",
-      "You may need to manually download and install these packages.",
-      wrap = FALSE
+      text,
+      "You may need to manually download and install these packages."
     )
 
   }
@@ -419,9 +379,9 @@ renv_hydrate_report <- function(packages, na, linkable) {
     }
 
     renv_pretty_print_records_pair(
+      preamble = preamble,
       old = list(),
       new = records,
-      preamble  = preamble,
       postamble = postamble,
       formatter = formatter
     )
@@ -430,9 +390,9 @@ renv_hydrate_report <- function(packages, na, linkable) {
 
   if (length(na)) {
     renv_pretty_print(
-      values    = csort(names(na)),
-      preamble  = "The following packages are used in this project, but not available locally:",
-      postamble = "renv will attempt to download and install these packages."
+      "The following packages are used in this project, but not available locally:",
+      csort(names(na)),
+      "renv will attempt to download and install these packages."
     )
   }
 
