@@ -3,10 +3,10 @@ renv_task_create <- function(callback, name = NULL) {
 
   # create name for task callback
   name <- name %||% as.character(substitute(callback))
-  id <- paste("renv", name, sep = ":::")
+  name <- paste("renv", name, sep = ":::")
 
   # remove an already-existing task of the same name
-  removeTaskCallback(id)
+  removeTaskCallback(name)
 
   # otherwise, add our new task
   addTaskCallback(
@@ -16,17 +16,6 @@ renv_task_create <- function(callback, name = NULL) {
 
 }
 
-renv_task_remove <- function(callback, name = NULL) {
-
-  # create name for task callback
-  name <- name %||% as.character(substitute(callback))
-  id <- paste("renv", name, sep = ":::")
-
-  # remove it
-  removeTaskCallback(id)
-
-}
-
 renv_task_callback <- function(callback, name) {
 
   force(callback)
@@ -36,9 +25,8 @@ renv_task_callback <- function(callback, name) {
 
     status <- tryCatch(callback(), error = identity)
     if (inherits(status, "error")) {
-      fmt <- "renv background task '%s' failed; it will be stopped"
-      msg <- sprintf(fmt, name)
-      warning(msg, call. = FALSE)
+      writef("Error in background task '%s': %s", name, conditionMessage(status))
+      writef("Background task '%s' will be stopped.", name)
       return(FALSE)
     }
 
@@ -51,6 +39,7 @@ renv_task_callback <- function(callback, name) {
 renv_task_unload <- function() {
   callbacks <- getTaskCallbackNames()
   for (callback in callbacks)
-    if (startswith(callback, "renv:::"))
-      removeTaskCallback(callback)
+    for (prefix in c("renv_", "renv:::"))
+      if (startswith(callback, prefix))
+        removeTaskCallback(callback)
 }
