@@ -18,7 +18,6 @@ unload <- function(project = NULL, quiet = FALSE) {
   renv_unload_envvars(project)
   renv_unload_sandbox(project)
   renv_unload_libpaths(project)
-  renv_unload_profile(project)
 
 }
 
@@ -27,8 +26,7 @@ renv_unload_shims <- function(project) {
 }
 
 renv_unload_project <- function(project) {
-  options(renv.project.path = NULL)
-  Sys.unsetenv("RENV_PROJECT")
+  renv_project_clear()
 }
 
 renv_unload_profile <- function(project) {
@@ -46,7 +44,3 @@ renv_unload_sandbox <- function(project) {
 renv_unload_libpaths <- function(project) {
   renv_libpaths_restore()
 }
-
-renv_unload_profile <- function(project) {
-  Sys.unsetenv("RENV_PROFILE")
-}
