@@ -4,8 +4,9 @@ renv_warnings_unknown_sources <- function(records) {
   if (empty(records))
     return(FALSE)
 
+  # TODO: Should this be documented?
   enabled <- renv_config_get(
-    name    = "unknown_sources",
+    name    = "unknown.sources",
     scope   = "warnings",
     type    = "logical[1]",
     default = TRUE
@@ -15,11 +16,11 @@ renv_warnings_unknown_sources <- function(records) {
     return(FALSE)
 
   renv_pretty_print_records(
-    records,
     "The following package(s) were installed from an unknown source:",
+    records,
     c(
       "renv may be unable to restore these packages in the future.",
-      "Consider re-installing these packages from a known source (e.g. CRAN)."
+      "Consider reinstalling these packages from a known source (e.g. CRAN)."
     )
   )
 
