@@ -1,10 +1,7 @@
 
-context("Internal")
-
 test_that("R files have balanced covr exclusions", {
 
-  owd <- setwd("../..")
-  on.exit(setwd(owd), add = TRUE)
+  renv_scope_wd("../..")
 
   errors <- stack()
 
@@ -14,7 +11,7 @@ test_that("R files have balanced covr exclusions", {
     nocov <- FALSE
     contents <- catch(readLines(file))
     if (inherits(contents, "error")) {
-      vwritef("[%s]: %s", file, conditionMessage(contents))
+      writef("[%s]: %s", file, conditionMessage(contents))
       return()
     }
 
